#!/usr/bin/perl
use warnings;
use strict;

##############################################
# SudokuSolver (c) Amelie Zapf 2006
#
# Takes a sudoku as a comma-separated list of
# rows as input, with blanks denoted by zero
# (e.g. 013074002,702800010,...), solves it
# while telling you what it's doing and prints
# the solution.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307, USA.
#############################################

my $sudoku=$ARGV[0];
my @sudoku=split /,/, $sudoku;
foreach (@sudoku) {
    my @tmp=split //, $_;
    $_=\@tmp;
}

my $solved=0;
my $maxIts=0;
my $changed=1;
my $guess=0;
my @attempts=();
my @attemptIts=();

&printSudoku(@sudoku);
while (!$solved && $maxIts<81) {
    if ($changed==0) { print "\nGuess needed.\n"; $guess=1; }
    $changed=0;
    $solved=1;
    $maxIts++;
    for (my $i=0; $i<9; $i++) {
	for (my $j=0; $j<9; $j++) {
	    if (!$sudoku[$i][$j]) {
                $solved=0;
		my @col=&column($i, $j, \@sudoku);
		my @row=&row($i, $j, \@sudoku);
		my @blk=&block($i, $j, \@sudoku);
		my @poss=&merge(\@col, \@row, \@blk);
		@poss=&negate(@poss);
		if (scalar @poss==1) { 
                    $changed=1;
		    $sudoku[$i][$j]=$poss[0];
		    print "Found $poss[0] at $i, $j.... ";
                } elsif (scalar @poss==0) {
		    if (scalar @attempts>0) {
		        print "\nCannot be solved. Reverting guess.\n";
		        my $tmp=pop @attempts;
		        $maxIts=pop @attemptIts;
		        @sudoku=&decatenate($tmp);
                    } else {
			die "\nIll-defined sudoku. Cannot be " .
	   		    "solved at all.\n";
		    }
		} elsif (scalar @poss==2 && $guess==1) {
		    $changed=1;
		    $guess=0;
		    $sudoku[$i][$j]=$poss[1];
		    my $tmp=&catenate(@sudoku);
		    push @attempts, ($tmp);
                    push @attemptIts, ($maxIts);
		    $sudoku[$i][$j]=$poss[0];
		    print "Guessed $poss[0] (alternative $poss[1])" .
			" for $i, $j.\n";
		}
	    }
        }
    }
}
print "\n";
&printSudoku(@sudoku);

sub catenate {
    my @array=@_;
    foreach (@array) {
	my @tmp=@$_;
 	$_=join "t", @tmp;
    }
    my $string=join "n", @array;
    return $string;
}

sub decatenate {
    my $string=$_[0];
    my @string=split /n/, $string;
    foreach (@string) {
	my @tmp=split /t/, $_;
	$_=\@tmp;
    }
    return @string;
}

sub printSudoku {
    my @sudoku=@_;
    foreach (@sudoku) {
	my @row=@$_;
	my $row=join "  ", @row;
	print $row . "\n";
    }
    return 1;
}

sub column {
    my ($i, $j, $sudokuptr)=@_;
    my @sudoku=@$sudokuptr;
    my @col=();
    for (my $run=0; $run<9; $run++) {
	if ($sudoku[$i][$run]) { push @col, ($sudoku[$i][$run]); }
    }
    return @col;
}

sub row {
    my ($i, $j, $sudokuptr)=@_;
    my @sudoku=@$sudokuptr;
    my @row=();
    for (my $run=0; $run<9; $run++) {
        if ($sudoku[$run][$j]) { push @row, ($sudoku[$run][$j]); }
    }
    return @row;	
}

sub block {
    my ($i, $j, $sudokuptr)=@_;
    my @sudoku=@$sudokuptr;
    my @block=();
    my $start1=$i-($i%3);
    my $start2=$j-($j%3);
    for (my $run1=$start1; $run1<$start1+3; $run1++) {
        for (my $run2=$start2; $run2<$start2+3; $run2++) {
            if ($sudoku[$run1][$run2]) { push @block, ($sudoku[$run1][$run2]); }
	}
    }
    return @block;
}

sub merge {
    my @listoflists=@_;
    my @newlist=();
    foreach (@listoflists) {
        my @tmp=@$_;
	push @newlist, @tmp;
    }
    @newlist=&uniquify(@newlist);
    return @newlist;
}

sub uniquify {
    my @list=@_;
    my @newlist=();
    foreach (@list) {
        if (!&contained($_, \@newlist)) { push @newlist, ($_); }
    }
    return @newlist;
}

sub negate {
    my @list=@_;
    my @newlist=();
    for (my $run=1; $run<10; $run++) {
	if (!&contained($run, \@list)) { push @newlist, ($run); }
    }
    return @newlist;
}

sub contained {
    my ($item, $listptr)=@_;
    my @list=@$listptr;
    foreach (@list) {
	if ($item eq $_) { return 1; last; }
    }
    return 0;
}

